<?php
/*
** @name: admin_img.php
**
** @description: controller for plan images administration
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/plugin/plan/inc/common_admin_plan.inc.php';
include_once LIB_PLAN_IMG;

try
{
	if($action == 'admin')
	{
		// Get objects by types name to have all container & links objects in one array
		$objectsByTypes = $plan->getListObjectsByTypeName(TRUE, $path);
		// For link format source and target ids from old naming to new (old params saved in DB are like obj_type_id.obj_id.'object' new param is only the object id example: 33object become 3)
		$regex = '/\d(\d+)object/';
		foreach($objectsByTypes['link'] as $key => $obj)
		{
			if(!empty($obj['params']['service']))
			{
				$obj['params']['service'] = intval(explode(",", $obj['params']['service'])[0]);
			}
			if(!is_numeric($obj['params']['source']))
			{
				$srcMatches = array();
				preg_match($regex, $obj['params']['source'], $srcMatches);
				if(isset($srcMatches[1]))
				{
					$objectsByTypes['link'][$key]['params']['source'] = $srcMatches[1];
				}
			}
			if(!is_numeric($obj['params']['target']))
			{
				$targMatches = array();
				preg_match($regex, $obj['params']['target'], $targMatches);
				if(isset($targMatches[1]))
				{
					$objectsByTypes['link'][$key]['params']['target'] = $targMatches[1];
				}
			}
		}
		$smarty->assign('objectsByTypes', $objectsByTypes);

		// Get Icons menu list
		$icons = Model_plan::getIconsByGroups($dbconnection, $lang);
		$smarty->assign('icons', $icons);

		// Get background images list

		// From default plan dir
		$commonBgImages = array();
		$commonBgImages = getBgImgFilesFromDir($path, DATA_PLANS);
		// Remove default bg img from list
		unset($commonBgImages[array_search(DEFAULT_BG_FILE, $commonBgImages)]);
		// Sort
		sort($commonBgImages);
		$smarty->assign('commonBgImages', $commonBgImages);
		$smarty->assign('commonBgImgDir', DATA_PLANS);

		// From plan id dir
		$linkedBgImages = array();
		$planDataDir = DATA_PLANS . 'plan' . $plan->getId() . '/';
		$isPlanDir = is_dir($path . $planDataDir);
		if($isPlanDir)
		{
			$linkedBgImages = getBgImgFilesFromDir($path, $planDataDir);
			sort($linkedBgImages);
		}
		$smarty->assign('linkedBgImages', $linkedBgImages);
		$smarty->assign('linkdedBgImgDir', $planDataDir);

		// Get selected bg img
		$selectedImg = '';
		// If url & not default folder to be sure there is a file on the url
		$planParamUrl = $plan->getParams('url');
		if(!empty($planParamUrl) && $planParamUrl != DATA_PLANS)
		{
			$selectedImg = $planParamUrl;
		}
		$smarty->assign('selectedImg', $selectedImg);
		$smarty->assign('defaultBgImg', DEFAULT_BG_FILE);
	}
	elseif($action == 'updatePlan')
	{
		if(!isset($_POST['confirmIdPlan']) || $_POST['confirmIdPlan'] != $planId)
		{
			throw new \Exception($lang['PLAN_ERROR_PLAN_PARAM'] . ": " . $lang['PLAN_ERROR_PLAN_ID']);
		}

		// Instance the plan
		$plan = Model_plan::instancePlan($dbconnection, $planId, $user->getId(), FALSE, FALSE);
		// Check if page can be displayed
		if(!$isUserPerms['admin'] || $plan->getCurrentUserAccess() != 'WRITE')
		{
			$smarty->display('no_access.html');
			exit(0);
		}

		// Plan common data (name, folder, users share, resources, refresh)
		$plan = setPlanCommonData($dbconnection, $lang, $plan, $_POST);

		// Plan size params
		if(!empty($_POST['plan_size']))
		{
			$planSize = json_decode($_POST['plan_size'], TRUE);

			if(is_numeric($planSize['width']) && is_numeric($planSize['height']))
			{
				$plan->setParam(array('size'), $planSize);
			}
		}

		// Plan img params
		if(!empty($_POST['bgImage']))
		{
			$plan->setParam(array('url'), $_POST['bgImage']);
		}
		else
		{
			$plan->setParam(array('url'), '');
		}

		// Update the plan in DB
		Model_plan::updatePlan($dbconnection, $user->getId(), $planId, $plan->getName(), json_encode($plan->getParams()), $plan->getFolderId(), $plan->getAccessUsersList());

		// Update or add objects type container
		if(isset($_POST['container_objects']) && !empty($_POST['container_objects']))
		{
			// $_POST['container_objects'] is empty only if JS function doesn't JSON.stringify it. If JS stringify an empty array, php function see it as not empty because it's json encoded
			$containerObjects = json_decode($_POST['container_objects'], TRUE);
			$newObjContainerIds = array();
			foreach($containerObjects as $objData)
			{
				if(!empty($objData['id']))
				{
					if(is_numeric($objData['id']))
					{
						// The id is valid for DB it's not beginning by 'tmp-'. The object already exist in DB we just update it
						$obj = Model_plan::instanceObjectPlans($dbconnection, $planId, array($objData['id']), FALSE)[$objData['id']];
						$obj->setParam(array('position'), $objData['params']['position']);
						$obj->setParam(array('size'), $objData['params']['size']);
						Model_plan::updateObject($dbconnection, $objData['id'], $planId, $objData['name'], json_encode($obj->getParams()), $objData['icon_id'], NULL);
					}
					else
					{
						// It's a 'tmp-' id so it's a new object. And save it's new DB ID into $newObjContainerIds where key is the tmp id. Used later for links
						$newObjContainerIds[$objData['id']] = Model_plan::addObject($dbconnection, $planId, $objData['name'], 'container', json_encode($objData['params']), $objData['icon_id'], NULL);
					}
				}
			}
		}
		else
		{
			throw new \Exception($lang['PLAN_ERROR_OBJ_SAVE']);
		}

		// Update or add objects type link
		if(!empty($_POST['link_objects']))
		{
			$linkObjects = json_decode($_POST['link_objects'], TRUE);
			foreach($linkObjects as $objData)
			{
				// Replace links data which contains obj container tmp id's by New DB id's
				if(array_key_exists($objData['source'], $newObjContainerIds))
				{
					$objData['source'] = $newObjContainerIds[$objData['source']];
				}
				if(array_key_exists($objData['target'], $newObjContainerIds))
				{
					$objData['target'] = $newObjContainerIds[$objData['target']];
				}
				// Set name
				$objData['name'] = $objData['source'] . 'object-' . $objData['target'] . 'object';

				// Other parameters
				if(empty($objData['options']['isDisableArrows']))
				{
					$objData['options']['isDisableArrows'] = FALSE;
				}
				if(empty($objData['options']['isDisableLabels']))
				{
					$objData['options']['isDisableLabels'] = FALSE;
				}

				if(is_numeric($objData['id']))
				{
					// The id is valid for DB it's not beginning by 'tmp-'. The object already exist in DB we just update it
					$obj = Model_plan::instanceObjectPlans($dbconnection, $planId, array($objData['id']), FALSE)[$objData['id']];
					$obj->setParam(array('service'), $objData['service']);
					$obj->setParam(array('source'), $objData['source']);
					$obj->setParam(array('target'), $objData['target']);
					$obj->setParam(array('points'), $objData['points']);
					$obj->setParam(array('isDisableArrows'), $objData['options']['isDisableArrows']);
					$obj->setParam(array('isDisableLabels'), $objData['options']['isDisableLabels']);
					Model_plan::updateObject($dbconnection, $objData['id'], $planId, $objData['name'], json_encode($obj->getParams()), NULL, NULL);
				}
				else
				{
					$objParams = array(
						'service' => $objData['service'],
						'source' => $objData['source'],
						'target' => $objData['target'],
						'points' => $objData['points'],
						'isDisableArrows' => $objData['options']['isDisableArrows'],
						'isDisableLabels' => $objData['options']['isDisableLabels']
					);
					// It's a 'tmp-' id so it's a new object.
					Model_plan::addObject($dbconnection, $planId, $objData['name'], 'link', json_encode($objParams), NULL, NULL);
				}
			}
		}
		else
		{
			throw new \Exception($lang['PLAN_ERROR_OBJ_SAVE']);
		}

		// Delete items
		if(isset($_POST['delete_objects']) && !empty($_POST['delete_objects']))
		{
			$deleteObjectsId = json_decode($_POST['delete_objects'], TRUE);
			foreach($deleteObjectsId as $key => $id)
			{
				// Check if not tmp object in list (new object not saved in DB yet)
				if(!is_numeric($id))
				{
					unset($deleteObjectsId[$key]);
				}
			}
			if(!empty($deleteObjectsId))
			{
				Model_plan::deleteObjects($dbconnection, $planId, $deleteObjectsId);
			}
		}
		else
		{
			throw new \Exception($lang['PLAN_ERROR_OBJ_DEL']);
		}

		// If no error display success message
		$_SESSION["plan_msg_success"] = $lang['PLAN_UPD_OK'];

		// Redirect to admin page
		$url = "admin_img.php?action=admin&id=" . $planId;

		ob_end_clean();
		header("Location: $url", TRUE);
		exit(0);
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	$errors[] = $e->getMessage();
}
$smarty->assign('errors', $errors);

$smarty->display($templatePath . 'admin_img.html');


function getBgImgFilesFromDir($path, $dirPath)
{
	$list = array();
	$dir = opendir($path . $dirPath);
	for($i=0; $f=readdir($dir); $i++)
	{
		if(is_file($path . $dirPath . $f))
		{
			$fileExt = preg_replace("#(.+)\.(.+)#", "$2", $f);
			$fileExt = strtolower($fileExt);
			if(in_array($fileExt, VALID_BG_FILE_EXT))
			{
				$list[] = $f;
			}
		}
	}
	closedir($dir);

	return $list;
}
?>
