<?php
/*
** @name: admin_rack.php
**
** @description: controller for plan rack administration
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/plugin/plan/inc/common_admin_plan.inc.php';
include_once LIB_PLAN_RACK;

define('RACK_LIMITS', array(
	'uSizeMin' => 2,
	'uSizeMax' => 48,
	'zoomMin' => 60,
	'zoomMax' => 200
));
define('RACK_ITEMS_TYPES_NOT_NODE', array("blank", "cable_manag", "patch_panel", "console", "shelf", "power_strip"));

try
{
	if($action == 'admin')
	{
		// Get plan objects devices name & for each rack U the ID of the object linked to it
		$objectsInfo = getRackItemsUlinkAndDeviceName($dbconnection, $plan->getListObjects());
		$smarty->assign('devicesName', $objectsInfo['devicesName']);
		$smarty->assign('uLinksIds', $objectsInfo['uLinksIds']);

		$smarty->assign('RACK_LIMITS', RACK_LIMITS);
		$smarty->assign('RACK_ITEMS_VALID_TYPES', RACK_ITEMS_VALID_TYPES);
		$smarty->assign('RACK_ITEMS_TYPES_NOT_NODE', RACK_ITEMS_TYPES_NOT_NODE);
	}
	elseif($action == 'updatePlan')
	{
		if(!isset($_POST['confirmIdPlan']) || $_POST['confirmIdPlan'] != $planId)
		{
			throw new \Exception($lang['PLAN_ERROR_PLAN_PARAM'] . ": " . $lang['PLAN_ERROR_PLAN_ID']);
		}

		// Instance the plan
		$plan = Model_plan::instancePlan($dbconnection, $planId, $user->getId(), FALSE, FALSE);
		// Check if page can be displayed
		if(!$isUserPerms['admin'] || $plan->getCurrentUserAccess() != 'WRITE')
		{
			$smarty->display('no_access.html');
			exit(0);
		}

		// Plan common data (name, folder, users share, resources, refresh)
		$plan = setPlanCommonData($dbconnection, $lang, $plan, $_POST);

		// Plan size params
		if(!empty($_POST['plan_size']))
		{
			$planSize = json_decode($_POST['plan_size'], TRUE);

			if(is_numeric($planSize['width']) && is_numeric($planSize['height']))
			{
				$plan->setParam(array('size'), $planSize);
			}
		}

		// Plan Rack params
		$rackParams = array();
		if(isset($_POST['rack_params']) && !empty($_POST['rack_params']))
		{
			parse_str(urldecode($_POST['rack_params']), $rackParams);
		}
		if(!isset($rackParams['empty_u_hidden']))
		{
			$rackParams['empty_u_hidden'] = "";
		}
		if(isset($rackParams['u_size']) && isset($rackParams['zoom']))
		{
			$plan->setParam(array('rack'), $rackParams);
		}
		else
		{
			throw new \Exception($lang['PLAN_ERROR_RACK'] . ": " . $lang['PLAN_ERROR_RACK_PARAM']);
		}

		// Update the plan in DB
		Model_plan::updatePlan($dbconnection, $user->getId(), $planId, $plan->getName(), json_encode($plan->getParams()), $plan->getFolderId(), $plan->getAccessUsersList());

		// Rack items
		if(isset($_POST['rack_objects']) && !empty($_POST['rack_objects']))
		{
			// $_POST['rack_objects'] is empty only if JS function doesn't JSON.stringify it. If JS stringify an empty array, php function see it as not empty because it's json encoded
			$rackObjectsParams = json_decode($_POST['rack_objects'], TRUE);
			foreach($rackObjectsParams as $objParam)
			{
				// Check if rack item params are valid
				if(isValidRackItem($objParam['common']['item_type'], $objParam['common']['item_subtype'], $objParam['common']['u_size']))
				{
					if($objParam['info']['isNew'])
					{
						// New rack items
						$commonParams = array('common' => $objParam['common']);
						Model_plan::addObject($dbconnection, $planId, $planId.'-rack_item', 'rack_item', json_encode($commonParams));
					}
					else
					{
						// Update rack items
						Model_plan::updateObjectParamJSON($dbconnection, $objParam['info']['id'], json_encode($objParam['common']), array('common'));
					}
				}
				else
				{
					throw new \Exception($lang['PLAN_ERROR_RACK'] . ": " . $lang['PLAN_ERROR_INVALID_ITEM'] . " ID=" . $objParam['info']['id'] . " (" . $objParam['common']['item_type'] . "-" . $objParam['common']['item_subtype'] . "-" . $objParam['common']['u_size'] . "U)");
				}
			}
		}
		else
		{
			throw new \Exception($lang['PLAN_ERROR_RACK'] . ": " . $lang['PLAN_ERROR_RACK_ITEMS']);
		}

		// Delete items
		if(isset($_POST['delete_objects']) && !empty($_POST['delete_objects']))
		{
			$deleteObjectsId = json_decode($_POST['delete_objects'], TRUE);
			if(!empty($deleteObjectsId))
			{
				Model_plan::deleteObjects($dbconnection, $planId, $deleteObjectsId);
			}
		}
		else
		{
			throw new \Exception($lang['PLAN_ERROR_RACK'] . ": " . $lang['PLAN_ERROR_RACK_ITEMS_DEL']);
		}

		// If no error display success message
		$_SESSION["plan_msg_success"] = $lang['PLAN_UPD_OK'];

		// Redirect to admin page
		$url = "admin_rack.php?action=admin&id=" . $planId;

		ob_end_clean();
		header("Location: $url", TRUE);
		exit(0);
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	$errors[] = $e->getMessage();
}
$smarty->assign('errors', $errors);

$smarty->display($templatePath . 'admin_rack.html');

?>
