<?php
/*
** @name: dialog_plan.php
**
** @description: controller for all plans common dialog windows
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/plugin/plan/inc/common_dialog_plan.inc.php';

try
{
	if($action == 'delPlan')
	{
		// Get plan user access
		$access = Model_plan::getPlanUserAccess($dbconnection, $planId, $user->getId());

		if(isset($_POST['confirmIdPlan']) && $_POST['confirmIdPlan'] == $planId)
		{
			/*
			** Validation form sended
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				throw new \Exception('Cannot delete this plan ! User privilege missing.');
			}
			// Authorize deletion only if the plan id from the confirmation form is the same than the requested plan id to delete
			Model_plan::deletePlan($dbconnection, $planId);

			// Delete data folder
			if(is_dir($path . DATA_PLANS . 'plan' . $planId))
			{
				rmdirRecursive($path . DATA_PLANS . 'plan' . $planId);
			}

			// Success message
			$_SESSION["plan_msg_success"] = strtoupper($_POST['name']) . ": " . $lang['PLAN_DELETE_SUCCESS'];

			// Back to folder parent (null = racine)
			$url = $foldersURL;
			if(!empty($_POST['folder_id']) && is_numeric($_POST['folder_id']))
			{
				$url .= "?id=" . $_POST['folder_id'];
			}

			echo json_encode(array('url'=> $url));
			exit(0);
		}
		else
		{
			/*
			** Display html content
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				$smarty->display('no_access.html');
				exit(0);
			}
			$smarty->assign('planId', $planId);
			$smarty->assign('planName', $_POST['name']);
			$smarty->assign('folderId', $_POST['folder_id']);
			$linkedObjs = Model_plan::getObjectsLinkedOnPlan($dbconnection, $planId);
			$smarty->assign('linkedObjs', $linkedObjs);
		}
	}
	elseif($action == 'serviceGraph')
	{
		// Open node status pop-up when ctrl+right-click on object linked to node.
		$service = NULL;
		if(!empty($_POST['serviceId']) && is_numeric($_POST['serviceId']))
		{
			$service = Model_esia::getServices($dbconnection, NULL, intval($_POST['serviceId']))[0];
			$service['common_name'] = Model_esia::getCommonName($lang, $service['name'], $service['check_plugin_name']);
		}

		if(empty($service))
		{
			throw new \Exception('Failed to retreive service data.');
		}

		if(isset($_POST['isIframe']) && $_POST['isIframe'] == TRUE)
		{
			$smarty->assign('isIframe', TRUE);
		}

		$smarty->assign('service', $service);
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	$errors[] = $e->getMessage();
}

if($isJSONResp == TRUE)
{
	echo json_encode(array('error' => $errors));
	exit(0);
}
else
{
	$smarty->assign('errors', $errors);
}

$smarty->display($templatePath . 'dialog_plan.html');

?>
