<?php
/*
** @name: display_plan.php
**
** @description: controller for plan common display ajax requests
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/common.php';
include_once $path . '/Model_adm.php';
include_once $path . '/plugin/plan/plan.conf.php';
include_once MODEL_PLAN;
include_once LIB_PLAN;

define('NAME_FORBIDDEN_CHARS', array(' ', '.', '"'));
define('NAME_REPLACEMENT_CHAR', '_');

// Action
if(isset($_GET['action']))
{
	$action = $_GET['action'];
}
elseif(isset($_POST['action']))
{
	$action = $_POST['action'];
}

try
{
	if($action == 'addObjectOnPlan')
	{
		if(isset($_POST['newObj']) && !empty($_POST['newObj']))
		{
			$obj = $_POST['newObj'];
			$object = json_decode($obj);
			$object->name = str_replace(NAME_FORBIDDEN_CHARS, NAME_REPLACEMENT_CHAR, trim($object->name));
			if(isset($object->type_name))
			{
				$type = $object->type_name;
			}
			else
			{
				$type = $object->type;
			}
			$newObjId = Model_plan::addObject($dbconnection, $_REQUEST['currentPlan'], $object->name, $type, json_encode($object->params), $object->icon);
			$newObj = Model_plan::instanceObjectPlans($dbconnection, $_REQUEST['currentPlan'], array($newObjId), FALSE)[$newObjId];
		}

		echo json_encode($newObj->toArray());
		exit(0);
	}
	elseif($action == 'getObject')
	{
		$obj = Model_plan::instanceObjectPlans($dbconnection, $_GET['id'], array($_GET['objId']), FALSE)[$_GET['objId']];

		echo json_encode($obj->toArray());
		exit(0);
	}
	elseif($action == 'delObj')
	{
		Model_plan::deleteObjects($dbconnection, $_GET['id'], $_GET['objId']);

		echo json_encode(array('success' => TRUE));
		exit(0);
	}
	elseif($action == 'updateObject')
	{
		if(!empty($_GET['id']) && !empty($_POST['type']))
		{
			if($_POST['type'] == 'resize')
			{
				$objSize = array(
					'width' => $_POST['width'],
					'height' => $_POST['height']
				);
				Model_plan::updateObjectParamJSON($dbconnection, $_POST['object'], json_encode($objSize), array('size'));

				echo json_encode(array('success' => TRUE));
				exit(0);
			}
			elseif($_POST['type'] == 'redrag')
			{
				// First get old position params to not eraze position.z
				$objPos = Model_plan::getObjectParamJSON($dbconnection, $_POST['object'], array('position'));
				$objPos['x'] = intval($_POST['left']);
				$objPos['y'] = intval($_POST['top']);

				Model_plan::updateObjectParamJSON($dbconnection, $_POST['object'], json_encode($objPos), array('position'));

				echo json_encode(array('success' => TRUE));
				exit(0);
			}
		}
	}
	elseif($action == 'getNodeServices')
	{
		$servicesList = getNodeServicesList($dbconnection, $lang, $_GET['nodeId']);

		echo json_encode($servicesList);
		exit(0);
	}
	elseif($action == 'details')
	{
		// Open node status pop-up when ctrl+right-click on object linked to node.
		$alerts = array();
		if(!empty($_GET['objId']))
		{
			$obj = Model_plan::instanceObjectPlans($dbconnection, $_GET['id'], array($_GET['objId']), FALSE)[$_GET['objId']];
			$alerts = Model_plan::getObjectAlerts($dbconnection, $user, $lang, $obj->getParams(), $obj->getTypeName(), $obj->getLinkPlanId());
		}

		if(!empty($alerts))
		{
			echo json_encode($alerts);
		}
		else
		{
			echo json_encode(null);
		}
		exit(0);
	}
	elseif($action == 'getObjPlanLink')
	{
		// If an object is linked to a plan get the controller path to this plan
		$planController = '';
		if(!empty($_GET['objId']))
		{
			$obj = Model_plan::instanceObjectPlans($dbconnection, $_GET['id'], array($_GET['objId']), FALSE)[$_GET['objId']];

			if(!empty($obj->getLinkPlanId()))
			{
				$objPlanType = Model_plan::getPlanTypes($dbconnection, NULL, $obj->getLinkPlanId())[0];

				echo json_encode(array(
					'controller' => $objPlanType['controller'] . '&id=' . $obj->getLinkPlanId()
				));
				exit(0);
			}
		}
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	echo json_encode(array('error' => $e->getMessage()));
}

exit(0);

?>
