/* Service Tool */
var OBJ_SERVICE_DEFAULT_TYPE = "state";

var OBJ_SERVICE_DEFAULT_OPTS = {
	value: {
		isDisplayValNode: false,
		isDisplayValName: true,
		isDisplayValStateColor: true,
	},
	state: {
		isDisplayStateNode: false,
		selDisplayStateServData: "name-state",
		selDisplayStateStateColor: "state",
	},
	message: {
		isDisplayMsgNode: false,
		isDisplayMsgName: false,
	},
};

// Open object acions box
function setObjServiceEventsHandler(container) {
	container.on("click", function() {
		if (container.find(".plan-obj-infos").length == 0) {
			editServiceObj(container);
		}
	});
}

// On plan view page load create service object saved in DB
function setObjectServiceOnPlan(obj, containerClasses="planObjView") {
	var plan = $("#plan" + glob_plan.id),
		container = $("<div>", {
			"id": obj.id.toString() + "object",
			"class": "objService objService-" + obj.params.sub_type + " " + containerClasses,
			"data-id": obj.id,
			"data-type": obj.type_name,
		});

	container.css({
		"opacity": 0, // Hide object until it has been completely created to avoid anoying blink (because of ajax request data fetch)
		"top": obj?.params?.position?.y || 0,
		"left": obj?.params?.position?.x || 0,
		"width": obj?.params?.size?.width || "",
		"height": obj?.params?.size?.height || "",
	});

	plan.append(container);

	var addTooltipTitleOnServiceObj = function(container) {
		addTooltipTitle($(".plan-service-name", container));
		addTooltipTitle($(".plan-service-data", container));
		addTooltipTitle($(".plan-service-content", container));
	};

	switch (obj?.params?.sub_type) {
		case "value":
			setObjServiceValuesPromise(obj).then(function() {
				setObjServiceValDisplay(obj.id, obj.params?.options);
				setObjServiceStyles(obj.id, obj.params?.style);
				container.css("opacity", ""); // Show object when all elements have been updated
				addTooltipTitleOnServiceObj(container);
			});
			break;
		case "state":
			setObjServiceStatePromise(obj).then(function() {
				setObjServiceStateDisplay(obj.id, obj.params?.options);
				setObjServiceStyles(obj.id, obj.params?.style);
				container.css("opacity", ""); // Show object when all elements have been updated
				addTooltipTitleOnServiceObj(container);
			});
			break;
		case "message":
			setObjServiceMessagePromise(obj).then(function() {
				setObjServiceMessageDisplay(obj.id, obj.params?.options);
				setObjServiceStyles(obj.id, obj.params?.style);
				container.css("opacity", ""); // Show object when all elements have been updated
				addTooltipTitleOnServiceObj(container);
			});
			break;
		default:
			// Unknown type
			var error = glob_lang.PLAN_TOOLS_SERVICE_ERR_DISPLAY + obj.id.toString() + ". " + glob_lang.PLAN_TOOLS_SERVICE_ERR_TYPE + obj.type;
			displayUserMessage(error, true);
	}

	// Only for plan view mode obj
	if ($('.plan-main').attr('data-action') == 'plan') {
		// Open node state table on ctrl + click
		container.on("click", function(e) {
			if (e.ctrlKey) {
				getObjectsDetails(obj.id, glob_lang.PLAN_OBJECT_SERVICE + " - " + obj.id);
			}
		});
	}
}

// On page load create service object saved in DB
function adm_setObjectServiceOnPlan(obj) {
	setObjectServiceOnPlan(obj, "planObj");

	var container = $("#" + obj.id + "object");

	dragPlanToolObj(container);
	setObjServiceEventsHandler(container);
}

// Open service object configuration dialog
function configObjService(objId) {
	var url = glob_apache_path + "control_tools.php?action=confObjService&id=" + glob_plan.id + "&plan=" + $("#currentPlan").data("id") + "&objId=" + objId + "&defOpts=" + JSON.stringify(OBJ_SERVICE_DEFAULT_OPTS);

	openConfObjDialog(url, glob_lang.PLAN_TOOLS_SERVICE_CONF);
}

// Set service object resizable
function setResizeServiceObj(plan, container) {
	var box = $(".plan-service-box", container);

	container.resizable({
		disabled: false,
		containment: plan,
		minWidth: parseInt(box.css("font-size")) * 1.75,
		minHeight: parseInt(box.css("font-size")) * 1.5,
		start: function(e, ui) {
			// Handle sizes gap for border if in box-sizing: border-box
			var resizeHeightCorrection = box.outerHeight() - box.height(),
				resizeWidthCorrection = box.outerWidth() - box.width();

			ui.size.height += resizeHeightCorrection;
			ui.size.width += resizeWidthCorrection;
		},
	});
}

// Create object actions menu
function editServiceObj(container) {
	var planId = $("#currentPlan").data("id"),
		objId = container.data("id"),
		actions = $("<div>", {"class": "plan-obj-actions"}),
		closeButton = $("<button>", {"class": "obj-action close-action", "type": "button", "title": glob_lang.PLAN_CLOSE}),
		modifyButton = $("<button>", {"class": "obj-action modify-action", "type": "button", "title": glob_lang.PLAN_MODIFY}),
		delButton = $("<button>", {"class": "obj-action delete-action", "type": "button", "title": glob_lang.PLAN_DELETE}),
		resizeButton = $("<button>", {"class": "obj-action resize-action", "type": "button", "title": glob_lang.PLAN_RESIZE}),
		cloneButton = $("<button>", {"class": "obj-action clone-action", "type": "button", "title": glob_lang.PLAN_CLONE}),
		originalSize = {width: container.outerWidth(), height: container.outerHeight()};

	actions.append(modifyButton).append(cloneButton);
	if (container.hasClass("objService-message")) {
		// Resize action only for sub_type message
		actions.append(resizeButton);
	}
	actions.append(delButton).append(closeButton);
	initObjectEditMenu(container, actions);

	// Close
	closeButton.on("click", function() {
		closeObjectEditMenu(container);
	});

	// Modify parameters
	modifyButton.on("click", function() {
		if ($.isNumeric(planId)) {
			configObjService(objId);
			closeObjectEditMenu(container);
		}
	});

	// Delete object
	delButton.on("click", function() {
		actionDeleteObject(objId, actions);
	});

	var fctOnResizeRefused = function() {
		// Reset sizes
		container.css({
			"width": originalSize.width,
			"height": originalSize.height,
		});
	};

	// Resize object
	resizeButton.on("click", function() {
		setResizeServiceObj($('#plan' + planId), container);
		actionResizeObject(objId, actions, container, true, fctOnResizeRefused);
	});

	// Clone object
	cloneButton.on("click", function(e) {
		closeObjectEditMenu(container);
		getObjectPromise(objId).then(function(obj) {
			duplicateObjByClick(e, obj, adm_setObjectServiceOnPlan);
		});
	});
}

// After creation of a new object set it on the plan and initialise somme actions
function initNewObjService(newObj) {
	adm_setObjectServiceOnPlan(newObj);
	// Directly open edit parameters
	configObjService(newObj.id);
}

// Create new service object from tool bar
function createNewObjService(clickPos) {
	var planId = glob_plan.id,
		position = {
			x: clickPos.x,
			y: clickPos.y,
			z: 1,
		},
		size = {
			width: 0, // Will be set to auto
			height: 0,
		},
		params = {
			position: position,
			size: size,
			sub_type: OBJ_SERVICE_DEFAULT_TYPE,
			device_id: null,
			options: OBJ_SERVICE_DEFAULT_OPTS[OBJ_SERVICE_DEFAULT_TYPE],
			values: [],
			services: [],
		},
		newObj = {
			id: 'tmp-' + Date.now(),
			type_name: "service",
			name: planId + "-service",
			params: params,
		};

	createNewToolObj(newObj, planId, initNewObjService);
}

// Set the curent values of the styles fields
function planServiceSetFieldStylesValues(style) {
	$("#titleFontSize").val(style.title.fontSize);
	$("#titleFontColor").val("#" + style.title.fontColor);
	$("#titleBgColor").val("#" + style.title.bgColor);
	$("#titleBgOpacity").val(style.title.bgOpacity);

	$("#contentFontSize").val(style.content.fontSize);
	$("#contentFontColor").val("#" + style.content.fontColor);
	$("#contentBgColor").val("#" + style.content.bgColor);
	$("#contentBgOpacity").val(style.content.bgOpacity);

	$("#borderColor").val(style.box.borderColor);
	$("#borderWidth").val(style.box.borderWidth);
	$("#topLeftRad").val(style.box.borderRadTopLeft);
	$("#topRightRad").val(style.box.borderRadTopRight);
	$("#bottLeftRad").val(style.box.borderRadBottomRight);
	$("#bottRightRad").val(style.box.borderRadBottomLeft);
	$("#topPad").val(style.box.padTop);
	$("#leftPad").val(style.box.padRight);
	$("#rightPad").val(style.box.padBottom);
	$("#bottPad").val(style.box.padLeft);
}

// Get the curent values of the styles fields
function planServiceGetFieldStylesValues() {
	var title = {
			"fontColor": $("#titleFontColor").val().replace("#", ""),
			"fontSize": $("#titleFontSize").val(),
			"bgColor": $("#titleBgColor").val().replace("#", ""),
			"bgOpacity": $("#titleBgOpacity").val(),
		},
		content = {
			"fontColor": $("#contentFontColor").val().replace("#", ""),
			"fontSize": $("#contentFontSize").val(),
			"bgColor": $("#contentBgColor").val().replace("#", ""),
			"bgOpacity": $("#contentBgOpacity").val(),
		},
		box = {
			"borderColor": $("#borderColor").val().replace("#", ""),
			"borderWidth": $("#borderWidth").val(),
			"borderRadTopLeft": $("#topLeftRad").val(),
			"borderRadTopRight": $("#topRightRad").val(),
			"borderRadBottomRight": $("#bottRightRad").val(),
			"borderRadBottomLeft": $("#bottLeftRad").val(),
			"padTop": $("#topPad").val(),
			"padRight": $("#rightPad").val(),
			"padBottom": $("#bottPad").val(),
			"padLeft": $("#leftPad").val(),
		},
		style = {
			"title": title,
			"content": content,
			"box": box,
		};

	return style;
}

// Display styles update in real time on preview window
function planServicePrevLiveStyles(objId) {
	var container = $(".plan-preview #" + objId + "object");

	if (GLOBAL_OBJ.params.style == undefined) {
		GLOBAL_OBJ.params.style = {
			title: {},
			content: {},
			box: {},
		}
	}

	// Title
	$("#titleFontSize").on("change", function() {
		$(".plan-service-title", container).css("font-size", this.value);
		GLOBAL_OBJ.params.style.title.fontSize = this.value;
	});
	$("#titleFontColor").on("change", function() {
		$(".plan-service-title", container).css("color", this.value);
		GLOBAL_OBJ.params.style.title.fontColor = this.value.replace("#", "");
	});
	$("#titleBgColor, #titleBgOpacity").on("change", function() {
		$(".plan-service-title", container).css("background-color", setRgba($("#titleBgColor").val(), $("#titleBgOpacity").val()));
		GLOBAL_OBJ.params.style.title.bgColor = $("#titleBgColor").val().replace("#", "");
		GLOBAL_OBJ.params.style.title.bgOpacity = $("#titleBgOpacity").val();
	});

	// Styles
	// Font
	$("#contentFontSize").on("change", function() {
		$(".plan-service-box", container).css("font-size", this.value);
		GLOBAL_OBJ.params.style.content.fontSize = this.value;
	});
	$("#contentFontColor").on("change", function() {
		$(".plan-service-box", container).css("color", this.value);
		GLOBAL_OBJ.params.style.content.fontColor = this.value.replace("#", "");
	});
	// BG
	$("#contentBgColor, #contentBgOpacity").on("change", function() {
		// Types node & state
		$(".plan-service-content", container).css("background-color", setRgba($("#contentBgColor").val(), $("#contentBgOpacity").val()));
		// Types service & value
		$(".plan-service-box", container).css("background-color", setRgba($("#contentBgColor").val(), $("#contentBgOpacity").val()));

		GLOBAL_OBJ.params.style.content.bgColor = $("#contentBgColor").val().replace("#", "");
		GLOBAL_OBJ.params.style.content.bgOpacity = $("#contentBgOpacity").val();
	});
	// Border
	$("#borderColor").on("change", function() {
		$(".plan-service-box", container).css("border-color", this.value);
		GLOBAL_OBJ.params.style.box.borderColor = this.value.replace("#", "");
	});
	$("#borderWidth").on("change", function() {
		$(".plan-service-box", container).css("border-width", this.value);
		GLOBAL_OBJ.params.style.box.borderWidth = this.value;
	});
	// Border radius
	$("#topLeftRad").on("change", function() {
		$(".plan-service-box", container).css("border-top-left-radius", this.value + "px");
		GLOBAL_OBJ.params.style.box.borderRadTopLeft = this.value;
		if ($(this).hasClass("overhide")) {
			$(".plan-service-box", container).css({
				"border-top-right-radius": this.value + "px",
				"border-bottom-left-radius": this.value + "px",
				"border-bottom-right-radius": this.value + "px",
			});
			GLOBAL_OBJ.params.style.box.borderRadTopRight = this.value;
			GLOBAL_OBJ.params.style.box.borderRadBottomLeft = this.value;
			GLOBAL_OBJ.params.style.box.borderRadBottomRight = this.value;
		}
	});
	$("#topRightRad").on("change", function() {
		$(".plan-service-box", container).css("border-top-right-radius", this.value + "px");
		GLOBAL_OBJ.params.style.box.borderRadTopRight = this.value;
	});
	$("#bottLeftRad").on("change", function() {
		$(".plan-service-box", container).css("border-bottom-left-radius", this.value + "px");
		GLOBAL_OBJ.params.style.box.borderRadBottomLeft = this.value;
	});
	$("#bottRightRad").on("change", function() {
		$(".plan-service-box", container).css("border-bottom-right-radius", this.value + "px");
		GLOBAL_OBJ.params.style.box.borderRadBottomRight = this.value;
	});
	// Padding
	$("#topPad").on("change", function() {
		$(".plan-service-box", container).css("padding-top", this.value);
		GLOBAL_OBJ.params.style.box.padTop = this.value;
		if ($(this).hasClass("overhide")) {
			$(".plan-service-box", container).css({
				"padding-left": this.value,
				"padding-right": this.value,
				"padding-bottom": this.value,
			});
			GLOBAL_OBJ.params.style.box.padLeft = this.value;
			GLOBAL_OBJ.params.style.box.padRight = this.value;
			GLOBAL_OBJ.params.style.box.padBottom = this.value;
		}
	});
	$("#leftPad").on("change", function() {
		$(".plan-service-box", container).css("padding-left", this.value);
		GLOBAL_OBJ.params.style.box.padLeft = this.value;
	});
	$("#rightPad").on("change", function() {
		$(".plan-service-box", container).css("padding-right", this.value);
		GLOBAL_OBJ.params.style.box.padRight = this.value;
	});
	$("#bottPad").on("change", function() {
		$(".plan-service-box", container).css("padding-bottom", this.value);
		GLOBAL_OBJ.params.style.box.padBottom = this.value;
	});
}

// Get styles from an object on plan and return it formatted as object parameter style for DB
function getObjServiceStyles(objId) {
	var objBox = $("#" + objId + "object .plan-service-box"),
		objTitle = $(".plan-service-title", objBox),
		objContent = $(".plan-service-content", objBox);

	if (!objBox.hasClass("display-node")) {
		// Get background color and opacity on the .plan-service-box. The background is set on .plan-service-box except in display-node mode
		objContent = objBox;
	}

	var title = {
			"fontColor": rgbToHex(objTitle.css("color")),
			"fontSize": parseInt(objTitle.css("font-size")),
			"bgColor": rgbToHex(objTitle.css("background-color")),
			"bgOpacity": getRgbaOpacity(objTitle.css("background-color")),
		},
		content = {
			"fontColor": rgbToHex(objBox.css("color")),
			"fontSize": parseInt(objBox.css("font-size")),
			"bgColor": rgbToHex(objContent.css("background-color")),
			"bgOpacity": getRgbaOpacity(objContent.css("background-color")),
		},
		box = {
			"borderColor": rgbToHex(objBox.css("border-top-color")),
			"borderWidth": parseInt(objBox.css("border-top-width")),
			"borderRadTopLeft": parseInt(objBox.css("border-top-left-radius")),
			"borderRadTopRight": parseInt(objBox.css("border-top-right-radius")),
			"borderRadBottomRight": parseInt(objBox.css("border-bottom-right-radius")),
			"borderRadBottomLeft": parseInt(objBox.css("border-bottom-left-radius")),
			"padTop": parseInt(objBox.css("padding-top")),
			"padRight": parseInt(objBox.css("padding-right")),
			"padBottom": parseInt(objBox.css("padding-bottom")),
			"padLeft": parseInt(objBox.css("padding-left")),
		},
		style = {
			"title": title,
			"content": content,
			"box": box,
		};

	return style;
}

// Set object type service styles
function setObjServiceStyles(objId, style) {
	var objBox = $("#" + objId + "object .plan-service-box"),
		objTitle = $(".plan-service-title", objBox)
		objContent = $(".plan-service-content", objBox);

	objTitle.css({
		"font-size": style?.title?.fontSize || "",
		"color": "#" + style?.title?.fontColor || "",
		"background-color": setRgba(style?.title?.bgColor || "", style?.title?.bgOpacity || ""),
	});
	objBox.css({
		"font-size": style?.content?.fontSize || "",
		"color": "#" + style?.content?.fontColor || "",
		"background-color": setRgba(style?.content?.bgColor || "", style?.content?.bgOpacity || ""),
		"border-color": setRgba(style?.box?.borderColor || "", 1),
		"border-width": style?.box?.borderWidth || "",
		"border-top-left-radius": style?.box?.borderRadTopLeft + "px" || 0,
		"border-top-right-radius": style?.box?.borderRadTopRight + "px" || 0,
		"border-bottom-right-radius": style?.box?.borderRadBottomRight + "px" || 0,
		"border-bottom-left-radius": style?.box?.borderRadBottomLeft + "px" || 0,
		"padding-top": style?.box?.padTop || "",
		"padding-right": style?.box?.padRight || "",
		"padding-bottom": style?.box?.padBottom || "",
		"padding-left": style?.box?.padLeft || "",
	});
	objContent.css({
		"background-color": setRgba(style?.content?.bgColor || "", style?.content?.bgOpacity || ""),
	});
}

// Handle saving new object configuration parameters
function updObjServiceEvent(objId) {
	$("#formUpdObjService").submit(function(e) {
		e.preventDefault();

		saveOrderSelectValues("order_service_val");
		saveOrderSelectValues("order_service_state");

		// Get obj size
		var prevObj = $(".plan-preview #" + objId + "object"),
			size = { // +1 for rounded values (ex: 6.4 px) We need to be sure that it can fit at least the size of it's content to not create text wrapping
				"width": prevObj.outerWidth() + 1,
				"height": prevObj.outerHeight() + 1,
			};

		$.ajax({
			type: "POST",
			url: glob_apache_path + "control_tools.php?action=updateObjService&id=" + glob_plan.id,
			data: {object_id: objId, form_data: $(this).serialize(), size: JSON.stringify(size)},
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true);
			} else {
				// Update object parameters on main window (outside of dialog iframe)
				window.parent.$("#" + data.id + "object").remove();
				window.parent.adm_setObjectServiceOnPlan(data);
				// Close dialog window
				window.parent.$("#dialog-info").dialog("close"); // dialog("close") method need to be called from main window
			}
		});
	});
}

// Show hide parameters inputs related to other selections
function onChangeDisplayParams(objId, subType) {
	// Switch sub_type on radioSelect
	$("input[type=radio][name=sub_type]").change(function() {
		subType = $(this).val();

		// Show only corresponding type menu
		$(".service-content-select").hide();
		$("#serv-content-select-" + subType).show();
		$(".service-display-params").hide();
		$("#serv-display-" + subType).show();

		if (subType == "message") {
			$(".styles-title .stylesTitleBg").hide();
		} else {
			$(".styles-title .stylesTitleBg").show();
		}
	});

	// Show hide title style on display option select
	$("#isDisplayValNode, #isDisplayStateNode").change(function() {
		if ($(this).is(":checked")) {
			$(".styles-title").show();
		} else {
			$(".styles-title").hide();
		}
	});
	// Message sub_type display option select
	$("#isDisplayMsgNode, #isDisplayMsgName").change(function() {
		// Message title is composed by node and name. If one of them is displayed show title parameters (only font)
		if ($("#isDisplayMsgNode").is(":checked") || $("#isDisplayMsgName").is(":checked")) {
			$(".styles-title").show();
		} else {
			$(".styles-title").hide();
		}
	});
	// Init - show hide title style on page load if the right checkbox is selected for the selected sub_type
	if (subType == "value") {
		$("#isDisplayValNode").trigger("change");
	} else if (subType == "state") {
		$("#isDisplayStateNode").trigger("change");
	} else if (subType == "message") {
		// No background only font styles for title on message sub_type
		$(".styles-title .stylesTitleBg").hide();
		$("#isDisplayMsgNode").trigger("change");
	}
}

// Handle the node and content selection changes
function onChangeContentSelect(obj) {
	var updateContentFieldFct = function(subType) {
		// Update content select options if empty
		if (subType == "value") {
			if ($('#rrdname').has('option').length == 0) {
				updSelectServiceValue(obj);
			}
			$("#order_service_val").trigger("sortupdate");
		} else if (subType == "state") {
			if (obj.params.device_id != null && $('#stateServices').has('option').length == 0) {
				updSelectService(obj.params.device_id, $("#stateServices"));
			}
			$("#order_service_state").trigger("sortupdate");
		} else if (subType == "message") {
			if (obj.params.device_id != null && $('#msgServices').has('option').length == 0) {
				updSelectService(obj.params.device_id, $("#msgServices"));
			} else {
				// Do it anyway but in else to not trigger it several times because it's triggered by updSelectService
				$("#msgServices").trigger("change");
			}
		}
	};

	// On page load, enable content field if node field is not empty
	if ($("#selectNode").val().length != 0) {
		$("#rrdname").prop("disabled", false);
		$("#rrdname").trigger("chosen:updated");
		$("#stateServices").prop("disabled", false);
		$("#stateServices").trigger("chosen:updated");
		$("#msgServices").prop("disabled", false);
		$("#msgServices").trigger("chosen:updated");
	}

	// Switch sub_type on radioSelect
	$("input[type=radio][name=sub_type]").change(function() {
		obj.params.sub_type = $(this).val();
		updateContentFieldFct(obj.params.sub_type);
	});

	// On change
	$("#selectNode").change(function() {
		// Update device_id
		obj.params.device_id = $("#selectNode").val();
		// Clean content fields for each sub types
		// value
		$("#rrdname").empty();
		$("#order_service_val").empty();
		$("#rrdname").prop('disabled', false);
		// state
		$("#stateServices").empty();
		$("#order_service_state").empty();
		$("#stateServices").prop('disabled', false);
		// message
		$("#msgServices").empty();
		$("#msgServices").prop('disabled', false);

		updateContentFieldFct(obj.params.sub_type);
	});

	// Reload preview on select or order change
	onSelectServiceValueUpdPrev(obj);
	onSelectServiceStateUpdPrev(obj);
	onSelectServiceMessageUpdPrev(obj);
}

// Init plan service object configuration
function initServiceConf(obj, planId, planController) {
	switchIconRadioSelect("sub_type");
	initOrderSelect("rrdname", "order_service_val");
	initOrderSelect("stateServices", "order_service_state");
	updObjServiceEvent(obj.id);
	initStyleActions();
	onChangeContentSelect(obj);
	onChangeDisplayParams(obj.id, obj.params.sub_type);

	createPlanPreviewPromise(obj, planId, planController).then(function() {
		var prevObj = $(".plan-preview #" + obj.id + "object");

		// If an object doesn't have styles saved in DB. Get the default styles applied in css on the preview and update form fields
		if (obj.params.style == undefined) {
			var style = getObjServiceStyles(obj.id);
			planServiceSetFieldStylesValues(style);
		}

		planServicePrevLiveStyles(obj.id);

		// Handle display parameters for each sub_type
		// Initial from object params
		if (obj.params.sub_type == "value") {
			setObjServiceValDisplay(obj.id, obj.params.options);
		} else if (obj.params.sub_type == "state") {
			setObjServiceStateDisplay(obj.id, obj.params.options);
		} else if (obj.params.sub_type == "message") {
			setObjServiceMessageDisplay(obj.id, obj.params.options);
		}
		// Set on change events
		updtPreviewServiceValDisplay(obj.id);
		updtPreviewServiceStateDisplay(obj.id);
		updtPreviewServiceMessageDisplay(obj.id);

		// Refresh display on sub_type change
		$("input[type=radio][name=sub_type]").change(function() {
			var subType = $(this).val();

			// Change object type class
			prevObj.removeClass("objService-value objService-state objService-message");
			prevObj.addClass("objService-" + subType);
			// Remove all display classes
			$(".plan-service-box", prevObj).removeClass().addClass("plan-service-box");
		});

		// Refresh preview content size when it's updated
		// $(".plan-preview").trigger("contentupdated");
		$(".plan-preview").on("contentupdated", function(e) {
			updatePreviewcontentSize(prevObj.outerWidth(), prevObj.outerHeight());
		});
	});
}

// Init plan service_value in tool bar
function initPlanToolService() {
	// Safety if count var hasn't been initilised. It's used to create temporary objects with differents ID's
	if (typeof count === "undefined") {
		count = 0;
	}

	$(".plan-tool-service").on("click", function(e) {
		createObjByClick(e, "cell", createNewObjService);
	});
}

// Update the values selection options
function updSelectServiceValue(obj) {
	if (obj.params.device_id != null) {
		$.ajax({
			type: "GET",
			url: glob_apache_path + "control_tools.php?id=" + glob_plan.id,
			data: {action: "getNodeServicesValNames", nodeId: obj.params.device_id},
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true);
			} else {
				$.each(data, function(key, rrd) {
					$("#rrdname").append("<option value='" + rrd.value + "'>" + rrd.name + "</option>");
				});
				$("#rrdname").trigger("chosen:updated");
			}
		});
	}
}

// Update preview and style fields on change type object completed. Will apply default css values for specific type if no style params
function getServiceObjTypeStyles() {
	setObjServiceStyles(GLOBAL_OBJ.id, GLOBAL_OBJ.params.style);
	var prevStyles = getObjServiceStyles(GLOBAL_OBJ.id);
	planServiceSetFieldStylesValues(prevStyles);
}

/***** TYPE VALUE *****/

// Set object type service_value data content
function setObjServiceValues(objId, params) {
	// Default params if not setted
	if (params.device_name == undefined) {
		params.device_name = glob_lang.NODE;
	}
	if (params?.values.length == 0) {
		params["values"][0] = {
			name: glob_lang.PLAN_SERVICE_VAL_DEFAULT_NAME,
			state: 3,
			unit: "",
			value: "0"
		};
	}

	var container = $("#" + objId + "object"),
		table = $("<table>", {"class": "plan-service-box", "html":
			'<thead><tr"><td colspan="100%" class="plan-service-title">' + params.device_name + '</td></tr></thead>'+
			'<tbody class="plan-service-content"></tbody>'
		});

	container.empty(); // Clean obj content
	container.append(table);

	// Create table row for each service value
	$.each(params.values, function(key, val) {
		if (val.unit.length > 0) {
			// If value unit is not empty display it with a space between it and value
			val.unit = " " + val.unit;
		}

		var tRow = $("<tr>", {"html":
				'<td class="plan-service-name" title="' + params.device_name + '">' + val.name +'</td>'+
				'<td class="plan-service-data" title="' + params.device_name + ' > ' + val.name + '">' + val.value + val.unit  +'</td>'
			});

		// Set color state color
		if (PLAN_VALID_SERVICE_STATE[val.state] == undefined) {
			val.state = 3;
		}
		var stateColor = PLAN_SERVICE_STATE_COLORS[val.state];

		tRow.find(".plan-service-data").css({"background-color": stateColor});

		$(".plan-service-content", table).append(tRow);
	});
}

// Set object type service_value data content. It's a promise to allow performing some tasks when ajax .done function is finish
function setObjServiceValuesPromise(obj) {
	if (obj.params?.values == undefined) {
		obj.params.values = [];
	}

	if (obj.params.device_id == null || obj.params.values.length == 0) {
		// Return empty promise to not wait for an ajax response
		return new Promise(function(resolve, reject) {
			setObjServiceValues(obj.id, obj.params); // Display default obj values
			resolve();
		});
	} else {
		// Return ajax promise
		return $.ajax({
			type: "GET",
			url: glob_apache_path + "control_tools.php?id=" + glob_plan.id,
			data: {action: "getNodeServicesValues", nodeId: obj.params.device_id, values: obj.params.values},
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) { // Error Message
				displayUserMessage(data.error, true);
				setObjServiceValues(obj.id, obj.params); // Display default obj values if data fetch error
			} else {
				setObjServiceValues(obj.id, data);
			}
		});
	}
}

// Set display classes in function of options selected
function setObjServiceValDisplay(objId, opts) {
	var box = $("#" + objId + "object .plan-service-box");

	if (opts?.isDisplayValNode) {
		box.addClass("display-node");
	} else {
		box.removeClass("display-node");
	}

	if (opts?.isDisplayValStateColor) {
		box.addClass("display-state");
	} else {
		box.removeClass("display-state");
	}

	if (opts?.isDisplayValName) {
		box.addClass("display-name");
	} else {
		box.removeClass("display-name");
	}
}

// Set object service type value display layout on plan preview when user change parameters
function updtPreviewServiceValDisplay(objId) {
	var obj = $("#" + objId + "object");

	$("#isDisplayValNode").change(function() {
		if ($(this).is(":checked")) {
			$(".plan-service-box", obj).addClass("display-node");
		} else {
			$(".plan-service-box", obj).removeClass("display-node");
		}
	});

	$("#isDisplayValStateColor").change(function() {
		if ($(this).is(":checked")) {
			$(".plan-service-box", obj).addClass("display-state");
		} else {
			$(".plan-service-box", obj).removeClass("display-state");
		}
	});

	$("#isDisplayValName").change(function() {
		if ($(this).is(":checked")) {
			$(".plan-service-box", obj).addClass("display-name");
		} else {
			$(".plan-service-box", obj).removeClass("display-name");
		}
	});
}

// Update preview on value select or order change
function onSelectServiceValueUpdPrev(obj) {
	$("#order_service_val").on("sortupdate", function(e, ui) {
		var selectedValues = getOrderSelectValues("order_service_val"),
			values = [];

		$.each(selectedValues, function(order, val) {
			var rrdArray = val.split("-");

			values[order] = {id: rrdArray[0], name: rrdArray[1], order: order};
		});

		// Update values for preview reload
		obj.params.values = values;
		setObjServiceValuesPromise(obj).then(function() {
			$("#isDisplayValNode, #isDisplayValStateColor, #isDisplayValName").trigger("change");
			getServiceObjTypeStyles();
			// Update preview content size scroll or not
			$(".plan-preview").trigger("contentupdated");
		});
	});
}

/***** TYPE STATE *****/

// Set object type service_state data content
function setObjServiceState(objId, params) {
	// Default params if not setted
	if (params.device_name == undefined) {
		params.device_name = glob_lang.NODE;
	}
	if (params?.services.length == 0) {
		params["services"][0] = {
			name: glob_lang.PLAN_SERVICE_DEFAULT_NAME,
			state: 3,
			message: glob_lang.PLAN_MESSAGE,
		};
	}

	var container = $("#" + objId + "object"),
		table = $("<table>", {"class": "plan-service-box", "html":
			'<thead><tr"><td colspan="100%" class="plan-service-title">' + params.device_name + '</td></tr></thead>'+
			'<tbody class="plan-service-content"></tbody>'
		});

	container.empty(); // Clean obj content
	container.append(table);

	// Create table row for each service
	$.each(params.services, function(key, srv) {
		var stateText = getPlanServiceStateLegend(glob_lang, srv.state),
			tRow = $("<tr>", {"html":
				'<td class="plan-service-name" title="' + params.device_name + '">' + srv.name +'</td>'+
				'<td class="plan-service-data" title="' + params.device_name + ' > ' + srv.name + '">' + stateText  +'</td>'
			});

		// Set color state color
		if (PLAN_VALID_SERVICE_STATE[srv.state] == undefined) {
			srv.state = 3;
		}
		var stateColor = PLAN_SERVICE_STATE_COLORS[srv.state];

		tRow.find(".plan-service-name").css({"background-color": stateColor});
		tRow.find(".plan-service-data").css({"background-color": stateColor});

		$(".plan-service-content", table).append(tRow);
	});
}

// Set object type service_state data content. It's a promise to allow performing some tasks when ajax .done function is finish
function setObjServiceStatePromise(obj) {
	if (obj.params?.services == undefined) {
		obj.params.services = [];
	}

	if (obj.params.device_id == null || obj.params.services.length == 0) {
		// Return empty promise to not wait for an ajax response
		return new Promise(function(resolve, reject) {
			setObjServiceState(obj.id, obj.params); // Display default obj values
			resolve();
		});
	} else {
		// Return ajax promise
		return $.ajax({
			type: "GET",
			url: glob_apache_path + "control_tools.php?id=" + glob_plan.id,
			data: {action: "getNodeServicesData", nodeId: obj.params.device_id, services: obj.params.services},
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) { // Error Message
				displayUserMessage(data.error, true);
				setObjServiceState(obj.id, obj.params); // Display default obj values if data fetch error
			} else {
				setObjServiceState(obj.id, data);
			}
		});
	}
}

// Set display classes in function of options selected
function setObjServiceStateDisplay(objId, opts) {
	var box = $("#" + objId + "object .plan-service-box");

	if (opts?.isDisplayStateNode) {
		box.addClass("display-node");
	} else {
		box.removeClass("display-node");
	}

	switch (opts?.selDisplayStateStateColor) {
		case "name":
			box.removeClass("display-statec-state");
			box.addClass("display-statec-name");
			break;
		case "state":
			box.removeClass("display-statec-name");
			box.addClass("display-statec-state");
			break;
		case "name-state":
			box.addClass("display-statec-name");
			box.addClass("display-statec-state");
			break;
		default:
			box.removeClass("display-statec-name");
			box.removeClass("display-statec-state");
	}

	switch (opts?.selDisplayStateServData) {
		case "name":
			box.removeClass("display-data-state");
			box.addClass("display-data-name");
			break;
		case "state":
			box.removeClass("display-data-name");
			box.addClass("display-data-state");
			break;
		case "name-state":
			box.addClass("display-data-name");
			box.addClass("display-data-state");
			break;
		default:
			box.removeClass("display-data-name");
			box.removeClass("display-data-state");
	}
}

// Set object service type state display layout on plan preview when user change parameters
function updtPreviewServiceStateDisplay(objId) {
	var obj = $("#" + objId + "object");

	$("#isDisplayStateNode").change(function() {
		if ($(this).is(":checked")) {
			$(".plan-service-box", obj).addClass("display-node");
		} else {
			$(".plan-service-box", obj).removeClass("display-node");
		}
	});

	$("#selDisplayStateStateColor").change(function() {
		switch ($(this).val()) {
			case "name":
				$(".plan-service-box", obj).removeClass("display-statec-state");
				$(".plan-service-box", obj).addClass("display-statec-name");
				break;
			case "state":
				$(".plan-service-box", obj).removeClass("display-statec-name");
				$(".plan-service-box", obj).addClass("display-statec-state");
				break;
			case "name-state":
				$(".plan-service-box", obj).addClass("display-statec-name");
				$(".plan-service-box", obj).addClass("display-statec-state");
				break;
			default:
				$(".plan-service-box", obj).removeClass("display-statec-name");
				$(".plan-service-box", obj).removeClass("display-statec-state");
		}
	});

	$("#selDisplayStateServData").change(function() {
		switch ($(this).val()) {
			case "name":
				$(".plan-service-box", obj).removeClass("display-data-state");
				$(".plan-service-box", obj).addClass("display-data-name");
				break;
			case "state":
				$(".plan-service-box", obj).removeClass("display-data-name");
				$(".plan-service-box", obj).addClass("display-data-state");
				break;
			case "name-state":
				$(".plan-service-box", obj).addClass("display-data-name");
				$(".plan-service-box", obj).addClass("display-data-state");
				break;
			default:
				$(".plan-service-box", obj).removeClass("display-data-name");
				$(".plan-service-box", obj).removeClass("display-data-state");
		}
	});
}

// Update preview on service select or order change
function onSelectServiceStateUpdPrev(obj) {
	$("#order_service_state").on("sortupdate", function(e, ui) {
		var selectedServices = getOrderSelectValues("order_service_state"),
			services = [];

		$.each(selectedServices, function(order, id) {
			services[order] = {id: id, order: order};
		});

		// Update services for preview reload
		obj.params.services = services;
		setObjServiceStatePromise(obj).then(function() {
			$("#isDisplayStateNode, #selDisplayStateStateColor, #selDisplayStateServData").trigger("change");
			getServiceObjTypeStyles();
			// Update preview content size scroll or not
			$(".plan-preview").trigger("contentupdated");
		});
	});
}

/***** TYPE MESSAGE *****/

// Set object type service_message data content
function setObjServiceMessage(objId, params) {
	// Default params if not setted
	if (params.device_name == undefined) {
		params.device_name = glob_lang.NODE;
	}
	if (params?.services.length == 0) {
		params["services"][0] = {
			name: glob_lang.PLAN_SERVICE_DEFAULT_NAME,
			state: 3,
			message: glob_lang.PLAN_MESSAGE,
		};
	}

	var srv = params.services[0],
		container = $("#" + objId + "object"),
		box = $("<div>", {"class": "plan-service-box", "html":
				'<p class="plan-service-content" title="' + params.device_name + ' > ' + srv.name + '">'+
					'<span class="plan-service-title">'+
						'<span class="plan-service-node">' + params.device_name + '</span>'+
						'<span class="plan-service-name">' + srv.name + '</span>'+
					'</span>'+
					'<span class="plan-service-data">' + srv.message + '</span>'+
				'</p>'
		});

	container.empty(); // Clean obj content
	container.append(box);
}

// Set object type service_message data content. It's a promise to allow performing some tasks when ajax .done function is finish
function setObjServiceMessagePromise(obj) {
	if (obj.params.device_id == null || obj.params?.services == undefined) {
		obj.params.services = [];
	}

	if (obj.params.services.length == 0) {
		// Return empty promise to not wait for an ajax response
		return new Promise(function(resolve, reject) {
			setObjServiceMessage(obj.id, obj.params); // Display default obj values
			resolve();
		});
	} else {
		// Return ajax promise
		return $.ajax({
			type: "GET",
			url: glob_apache_path + "control_tools.php?id=" + glob_plan.id,
			data: {action: "getNodeServicesData", nodeId: obj.params.device_id, services: obj.params.services},
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) { // Error Message
				displayUserMessage(data.error, true);
				setObjServiceMessage(obj.id, obj.params); // Display default obj values if data fetch error
			} else {
				setObjServiceMessage(obj.id, data);
			}
		});
	}
}

// Set display classes in function of options selected
function setObjServiceMessageDisplay(objId, opts) {
	var box = $("#" + objId + "object .plan-service-box");

	if (opts?.isDisplayMsgNode) {
		box.addClass("display-node");
	} else {
		box.removeClass("display-node");
	}

	if (opts?.isDisplayMsgName) {
		box.addClass("display-name");
	} else {
		box.removeClass("display-name");
	}
}

// Set object service type message display layout on plan preview when user change parameters
function updtPreviewServiceMessageDisplay(objId) {
	var obj = $("#" + objId + "object");

	$("#isDisplayMsgNode").change(function() {
		if ($(this).is(":checked")) {
			$(".plan-service-box", obj).addClass("display-node");
		} else {
			$(".plan-service-box", obj).removeClass("display-node");
		}
	});

	$("#isDisplayMsgName").change(function() {
		if ($(this).is(":checked")) {
			$(".plan-service-box", obj).addClass("display-name");
		} else {
			$(".plan-service-box", obj).removeClass("display-name");
		}
	});
}

// Update preview on service select or order change
function onSelectServiceMessageUpdPrev(obj) {
	$("#msgServices").on("change", function() {
		// Update services for preview reload
		obj.params.services = [{id: $(this).val(), order: 0}];
		setObjServiceMessagePromise(obj).then(function() {
			$("#isDisplayMsgNode, #isDisplayMsgName").trigger("change");
			getServiceObjTypeStyles();
			// Update preview content size scroll or not
			$(".plan-preview").trigger("contentupdated");
		});
	});
}
