var OBJ_PREVIEW_CONTENT_MIN_GAP = 30;

// Return sate text legend
function getPlanServiceStateLegend(glob_lang, state) {
	switch (state) {
		case 0:
			return glob_lang.LGD_OK;
		case 1:
			return glob_lang.LGD_WARN;
		case 2:
			return glob_lang.LGD_CRI;
		case 4:
			return glob_lang.LGD_MAINTENANCE;
		default:
			return glob_lang.LGD_UNKNOWN;
	}
}

// Drag object on plan
function dragPlanToolObj(object)
{
	object.draggable({
		containment: $("#plan" + glob_plan.id),
		cursor: "move",
		stop: function(e, ui) {
			var objId = $(this).attr('data-id');

			if (objId !== undefined) {
				$.ajax({
					type: "POST",
					url: glob_apache_path + "display_plan.php?id=" + glob_plan.id,
					data: {action: "updateObject", type: "redrag", object: objId, left: ui.position.left, top: ui.position.top},
					dataType: "json"
				})
				.done(function(data) {
					if (data.error) { // Error Message
						displayUserMessage(data.error, true);
					}
					else if (!data.success) {
						displayUserMessage('Nothing updated', true);
					}
				});
			}
		},
	});

	multiDragPlanToolObj(object);
}

// Drag multiples plan tool objects together
function multiDragPlanToolObj(obj) {
	obj.on("multidrag:stop", function(e, ui) {
		var stop = $(this).draggable("option", "stop");

		if (stop !== null) {
			// Apply is used to pass element context as this
			stop.apply($(this), [e, ui]);
		}
	});
}

// Prevent to create an object outside of the plan limits
function checkObjectOnPlanLimits(plan, objectWidth, objectHeight, position) {
	var planWidth = plan[0].scrollWidth, // Full size with scroll content
		planHeight = plan[0].scrollHeight,
		maxRight = position.x + objectWidth,
		maxBottom = position.y + objectHeight;

	if (position.x < 4) { // Left
		position.x = 4;
	} else if (maxRight > planWidth) { // Right
		position.x = position.x - ((maxRight - planWidth) + 18);
	}
	if (position.y < 4) { // Top
		position.y = 4;
	} else if (maxBottom > planHeight) { // Bottom
		position.y = position.y - ((maxBottom - planHeight) + 18);
	}

	return position;
}

// Set plan tool styles when active
function setToolActive(plan, button, cursor="") {
	// Reset every other tools
	$(".plan-tools-bar .plan-tool").removeClass("active");
	$(".plan-tools-bar .plan-tool").css("cursor", "");

	// Initialize this tool
	$(".plan-tools-bar").css("cursor", cursor);
	plan.css("cursor", cursor);
	button.addClass("active");
	button.css("cursor", cursor);
}

// Reset plan tool styles when no longer active
function resetToolActive(plan, button) {
	$(".plan-tools-bar").css("cursor", "");
	plan.css("cursor", "");
	button.removeClass("active");
	button.css("cursor", "");
}

// Handle events for object creation on plan click
function createObjByClick(initialEvent, cursor="", fctSuccess) {
	var plan = $("#plan" + glob_plan.id),
		button = $(initialEvent.currentTarget);

	setToolActive(plan, button, cursor);

	getClickPositionOnPlan(initialEvent, plan)
		.then(function(position) {
			fctSuccess(position);
		})
		.catch(function(e) {
			// console.log(e);
		})
		.finally(function() {
			resetToolActive(plan, button);
		});
}

// Get click position on a plan without triggering any other event or action on plan elements. Must be use with .then to do next actions only when click event has been sended
// initialEvent => the start event wich will required a click position at next click
// plan => the plan where to fetch the click position
// return an object with y & x positons values
// Ex:
// getClickPositionOnPlan(e, $("#plan")).then(function(position) {
//	 console.log(position.y);
//	 console.log(position.x);
// });
function getClickPositionOnPlan(initialEvent, plan) {
	return new Promise(function(resolve, reject) {
		// Initialise new DOM element #click-capture-box at the same position & size than the plan & above it to capture the click position without triggering any other event or action on DOM element
		var captureBox = $("<div>", {"id": "click-capture-box"});

		// Set sizes values to fit all plan sizes in case of plan has scroll content
		captureBox.css({
			"width": plan[0].scrollWidth,
			"height": plan[0].scrollHeight,
		});

		if (!$("#click-capture-box").length) {
			// Create #click-capture-box if it doesn't exist yet
			plan.prepend(captureBox);
		}

		// Prevent the event click.clickcapture from applying on the event wich start this function to not auto-remove itself
		initialEvent.stopPropagation();
		// Reset click.clickcapture event if this function is re-called before it's done
		$(document).off("click.clickcapture");

		// Get click position
		$(document).one("click.clickcapture", function(e) {
			if (e.target.id === "click-capture-box") {
				var position = {
					"y": (e.pageY - $(e.target).offset().top) - 14,
					"x": (e.pageX - $(e.target).offset().left) - 8
				};

				resolve(position); // Resolve Promise (return function value)
			} else {
				reject("Wrong target clicked");
			}

			$("div#click-capture-box").remove();
		});

		$(document).on("keydown.abortclickcapture", function(e) {
			// Reset event if user press ESC on his keyboard
			if (e.keyCode == 27) {
				$(document).off("click.clickcapture");
				$("div#click-capture-box").remove();
				reject("Cancel");
			}
		});
	});
}

// Send a new object into the DB & if succeed, display it on the plan
function createNewToolObj(newObj, planId, callback) {
	var postObj = JSON.stringify(newObj);

	$.ajax({
		type: "POST",
		url: glob_apache_path + "display_plan.php?action=addObjectOnPlan&currentPlan=" + planId,
		data: {newObj: postObj, id: glob_plan.id},
		dataType: "json"
	})
	.done(function(data) {
		if (data.error) {
			// Error Message
			displayUserMessage(data.error, true);
		} else {
			// Create the new object on the plan
			callback(data);
		}
	})
	.fail(function(jqXHR, msg) {
		  displayUserMessage(msg, true);
	});
}

// Get object from the DB. Use it as promise => getObjectPromise(id).then(function(data) {});
function getObjectPromise(id) {
	return $.ajax({
		type: "GET",
		url: glob_apache_path + "display_plan.php?action=getObject",
		data: {id: glob_plan.id, objId: id},
		dataType: "json"
	}).done(function(data) {
		if (data.error) { // Error Message
			displayUserMessage(data.error, true);
		}
		else {
			return data;
		}
	});
}

// Duplicate an object formated for DB & set it on the plan by click
// initialEvent => the start event wich will required a click position at next click
function duplicateObjByClick(initialEvent, object, callback) {
	var plan = $("#plan" + glob_plan.id),
		planId = $("#currentPlan").data("id");

	plan.css("cursor", "copy");

	getClickPositionOnPlan(initialEvent, plan)
		.then(function(position) {
			// Verify object position on plan limits
			var planObj = $("#" + object.id + "object");

			position = checkObjectOnPlanLimits(plan, planObj.width(), planObj.height(), position);
			// Update duplicate object position with clicked & verifyed position
			object.params.position.y = position.y;
			object.params.position.x = position.x;

			// Create the new object
			createNewToolObj(object, planId, callback);

			plan.css("cursor", "");
		}).catch(function() {
			plan.css("cursor", "");
		});
}

// Converti une couleur RGB/RGBA en Hexa
function rgbToHex(color, addHashTag=false) {
	var match = color.replace(/\s/g, "").match(/^rgba?\((\d+),(\d+),(\d+)/i),
		output = color; // Si on ne match pas une couleur rgb ou rgba, on renvoie la variable passée.

	if (match && match.length === 4) {
		var rHex = ("0" + parseInt(match[1], 10).toString(16)).slice(-2),
			gHex = ("0" + parseInt(match[2], 10).toString(16)).slice(-2),
			bHex = ("0" + parseInt(match[3], 10).toString(16)).slice(-2);
		// Le résultat de match est transformée en entier puis en string base 16. On rajoute 0 devant pour être certain d'avoir un nombre sur 2 chiffres. ensuite on fait un slice -2 pour ne récupérer que les 2 chiffres de droite.

		output = rHex + gHex + bHex;
		if (addHashTag) {
			output = "#" + output;
		}
	}

	return output;
}

// Récupère l'opacité d'une couleur RGBA
function getRgbaOpacity(color) {
	var match = color.replace(/\s/g, "").match(/^rgba\(\d+,\d+,\d+,(\d?\.?\d?)/i),
		opacity = 1; // Si on ne match pas d'opacité dans la couleur, on retourne 1 car la couleur est complètement opaque.

	if (match && match.length === 2) {
		opacity = parseFloat("0" + match[1]);
		// "0" + est une sécurité pour toujours retourné un nombre meme si la valeur alpha est vide. Pour que parseFloat ne retourne pas NaN mais 0.
	}

	return opacity;
}

// Renvoie une couleur RGBA depuis une couleur hexa et/ou une opacité
function setRgba(hex, opacity=1) {
	var rgba = "", // Si on ne trouve pas de couleur, renvoie une chaine vide et donc aucune couleure ne sera appliquée
		cleanHex = hex.replace("#", ""),
		safeOpacity = parseFloat("0" + opacity), // Sécurité si opacity = "" opacity sera égale à 0
		match = [];

	if (cleanHex.length === 3) {
		// If hex shorthand value (ex: #fff)
		cleanHex = cleanHex + cleanHex;
	}
	match = cleanHex.match(/^([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})/i);

	if (match && match.length === 4) {
		var r = parseInt(match[1], 16),
			g = parseInt(match[2], 16),
			b = parseInt(match[3], 16);

		rgba = "rgba(" + r + "," + g + "," + b + "," + safeOpacity + ")";
	}

	return rgba;
}

// Set default object resizable
function setResizeObj(plan, obj, minWidth=100, minHeight=50) {
	obj.resizable({
		disabled: false,
		containment: plan,
		minWidth: minWidth,
		minHeight: minHeight,
	});
}

// Save new object size values
function saveResizedObj(objId, width, height) {
	if ($.isNumeric(objId)) {
		$.ajax({
			type: "POST",
			url: glob_apache_path + "display_plan.php?action=update&id=" + glob_plan.id + "&type=resize&object=" + objId + "&width=" + width + "&height=" + height,
			dataType: "html"
		})
		.done(function(data) {
			if (data != "null") { // Error Message
				displayUserMessage(data, true);
			}
		});
	}
}

// Handle the click on delete button of the object actions menu
function actionDeleteObject(objId, actionsMenu) {
	var acceptDelButton = $("<button>", {"class": "obj-action accept-action", "type": "button", "title": glob_lang.PLAN_DELETE_VALIDATE}),
		refuseDelButton = $("<button>", {"class": "obj-action refuse-action", "type": "button", "title": glob_lang.PLAN_DELETE_ABORT});

	acceptRefuseActionMenu(actionsMenu, acceptDelButton, refuseDelButton);

	// Validate object deletion
	acceptDelButton.on("click", function() {
		$.ajax({
			type: "GET",
			url: glob_apache_path + "display_plan.php",
			data: {action: "delObj", id: glob_plan.id, objId: objId},
			dataType: "json"
		})
		.done(function(data) {
			// Returned data contain the removed object ID. If data is empty, the object deletion has fail.
			if (data.error) { // Error Message
				displayUserMessage(data.error, true);
			}
			else if (data.success) {
				$("#" + objId + "object").remove();
			} else {
				displayUserMessage(glob_lang.PLAN_TOOLS_OBJECT_DELETE_FAILED, true);
			}
		});
	});
}

// Event to switch the selection on radio input
function switchIconRadioSelect(inputName) {
	// Get all the inputs radio with the same value name
	var radioSelects = $("input[type=radio][name=" + inputName + "]");

	// Set the initial active radio class
	$("input[type=radio][name=" + inputName + "]:checked").parent(".iconRadioSelect").addClass("active");

	radioSelects.change(function() {
		// Remove the class active on all iconRadioSelect wich are parents of a radioSelects
		radioSelects.parent(".iconRadioSelect").removeClass("active");
		// Add the class active on the current selected
		$(this).parent(".iconRadioSelect").addClass("active");
	});
}

// Open or close style-action inputs on click
function toggleStyleAction(container, forceOpen=false, forceClose=false) {
	var disabledImg = container.find(".style-action-disabled"),
		enabledImg = container.find(".style-action-enabled"),
		mainInput = container.find(".main-input"),
		othersInputs = $("input", container).not(".main-input");

	if (forceOpen) {
		container.addClass("active");
	} else if (forceClose) {
		container.removeClass("active");
	} else {
		container.toggleClass("active");
	}

	if (container.hasClass("active")) {
		disabledImg.hide();
		enabledImg.show();
		mainInput.removeClass("overhide");
		othersInputs.trigger("change");
	} else {
		enabledImg.hide();
		disabledImg.show();
		mainInput.addClass("overhide");
		// When closing set all input to main input value
		othersInputs.val(mainInput.val());
		othersInputs.trigger("change");
	}
}

// At styles config page load, expand or not inputs of style-action (Border radius, padding...). If all inputs have the same value, stay in one input display mode. If only one value is different expand all inputs.
function initStyleActions() {
	// Set expand on click event
	$(".style-action-btn").on("click", function() {
		toggleStyleAction($(this).closest(".style-action"), false, false);
	});

	// If inputs are not expanded, all inputs should have the same value than main-input
	$(".main-input").change(function() {
		if ($(this).hasClass("overhide")) {
			$(this).siblings("input").val($(this).val()).trigger("change");
		}
	});

	$(".style-action").each(function() {
		var container = $(this),
			mainVal = $(".main-input", container).val();

		$("input", container).each(function() {
			if ($(this).val() !== mainVal) {
				// On désactive les transitions (pas de transition pour la position au chargement du pop-up)
				$(".style-action-content", container).css("transition", "none");
				$("input", container).css("transition", "none");
				// On déploie les inputs
				toggleStyleAction(container, true, false);
				// On réactive les transitions (pour les changements au click)
				$(".style-action-content", container).css("transition", "");
				$("input", container).css("transition", "");

				return false;
			}
		});
	});
}

// Usefull when object is resized after the preview has been created
function updatePreviewcontentSize(width, height) {
	$(".plan-preview").css({
		"min-width": width + (OBJ_PREVIEW_CONTENT_MIN_GAP * 2),
		"min-height": height + (OBJ_PREVIEW_CONTENT_MIN_GAP * 2),
	});
}

// Create preview window for object configuration dialog
function createPlanPreviewPromise(obj, planId, planController) {
	// Get the plan from the controller
	return $.ajax({
		type: "GET",
		url: planController,
		data: {action: "dash_plan", id: planId},
		dataType: "html"
	})
	.done(function(data) {
		// Generate plan
		var prevWindow = $("#obj-" + obj.id + "-prev");
		// Don't show content until the preview has been finished to avoid view blinking and see content moving
		prevWindow.css("opacity", 0);
		// Set plan content into preview
		prevWindow.html(data);

		// Transform plan for preview
		var preview = $(".plan-main", prevWindow),
			plan = $("#plan" + planId, prevWindow),
			prevObj = $("#" + obj.id + "object", prevWindow),
			originalPlan = window.parent.$("#plan" + glob_plan.id),
			originalObj = $("#" + obj.id + "object", originalPlan),
			previewXPos = OBJ_PREVIEW_CONTENT_MIN_GAP - obj.params.position.x,
			previewYPos = OBJ_PREVIEW_CONTENT_MIN_GAP - obj.params.position.y,
			// Get sizes from the original objet in case of the object is created by ajax request it's sizes would be 0 here
			previewWidth = originalObj.outerWidth() + (OBJ_PREVIEW_CONTENT_MIN_GAP * 2),
			previewHeight = originalObj.outerHeight() + (OBJ_PREVIEW_CONTENT_MIN_GAP * 2);

		// Remove all classes from the .plan-main container & add the class for plan preview
		preview.removeClass().addClass("plan-main plan-preview");

		plan.css({
			"width": originalPlan[0].scrollWidth,
			"height": originalPlan[0].scrollHeight,
			"transform": "translate(" + previewXPos + "px, " + previewYPos + "px)"
		});

		preview.css({
			"min-width": previewWidth,
			"min-height": previewHeight
		});

		prevObj.css({
			"z-index": 300
		});
		// Show preview when finished
		prevWindow.css("opacity", "");
	});
}

// Delete element on a ordered select list
function deleteOrderSelectElement(selectId, item) {
	var value = item.attr("data-value");

	$("#" + selectId + " option[value=" + value +"]").attr("disabled", false);
	$("#" + selectId).trigger("chosen:updated");
	item.remove();
}

// Add element on a ordered select list
function addOrderSelectElement(selectId, listId) {
	var selected = $("#" + selectId + " option:selected"),
		val = selected.val();

	if (val?.length > 0) { // val is defined and not empty
		var text = selected.text(),
			li = $('<li data-value="' + val + '" class="block100change clearFix"><span>' + text + '</span><a class="delete-item" href="#"><img class="button" src="/theme/default/images/delete-dash.png" alt="delete"></a></li>');

		li.children(".delete-item").on("click", function(e) {
			deleteOrderSelectElement(selectId, li);
			$("#" + listId).trigger("sortupdate");
		});

		selected.attr("disabled", true).attr("selected", false);
		$("#" + selectId).trigger("chosen:updated");
		$("#" + listId).append(li);
		$("#" + listId).sortable("refresh");
	}
}

// Create sortable items selection
function initOrderSelect(selectId, listId) {
	$("#" + listId).sortable();

	$("#" + listId + " li").on("mousedown", function(e) {
		$(this).css("cursor", "grabbing");
	}).on("mouseup mouseleave", function(e) {
		$(this).css("cursor", "");
	});

	$(".order-select .delete-item").on("click", function(e) {
		deleteOrderSelectElement(selectId, $(this).closest("li"));
		$("#" + listId).trigger("sortupdate");
	});

	$("#add_" + listId).on("click", function(e) {
		addOrderSelectElement(selectId, listId);
		$("#" + listId).trigger("sortupdate");
	});
}

// Return order select values as an array
function getOrderSelectValues(listId) {
	var arr = [];

	// Create an array with key=order-position and value=select-option-value
	$("#" + listId + " li").each(function() {
		  arr.push($(this).attr("data-value"));
	});

	return arr;
}

// On form submit save values of order select into the hidden input
function saveOrderSelectValues(listId) {
	var arr = getOrderSelectValues(listId);

	// Save the result on an hidden input to retreive it in form values
	$("#input_" + listId).val(JSON.stringify(arr));
}
